/*
 * Copyright (C) 2007 JasperSoft http://www.jaspersoft.com
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed WITHOUT ANY WARRANTY; and without the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see http://www.gnu.org/licenses/gpl.txt 
 * or write to:
 * 
 * Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330,
 * Boston, MA  USA  02111-1307
 */

/** 
 *  \file jslist.c
 *  This sample shows how to list a directory in the repository.
 *  
 *  \see  jasperserver_ws_list

*/

#include <jasperserver.h>

const char server[] = "http://127.0.0.1:8080/jasperserver/services/repository";

/** 
 *  \brief Sample function to list a folder in the repository
 *
 *  \param dir URI of the folder to list.
*/
void list_directory(char *dir)
{
      jasperserver_request_t *request = NULL;
      jasperserver_server_t *jasperServer = NULL;
      jasperserver_resource_descriptor_t *res = NULL;
      jasperserver_operation_result_t *operationResult = NULL;
 
      // Create a new structure of type jasperserver_server_t to save the
      // server url and credentials.
      jasperServer = jasperserver_server_new();
      jasperserver_string_cset(jasperServer->url, server);
      jasperserver_string_cset(jasperServer->username, "tomcat");
      jasperserver_string_cset(jasperServer->password, "tomcat");
     
      // Create a new structure of type jasperserver_request_t to store
      // the request information like the resource to list    
      request = jasperserver_request_new();   
     
      //Creating and populating a new resource_descriptor...
      res = jasperserver_resource_descriptor_new();
      request->resource = res;
      jasperserver_string_cset(res->wsType, JS_TYPE_FOLDER);
      jasperserver_string_cset(res->uriString, dir);
      
      // Call the web service
      operationResult = jasperserver_ws_list(jasperServer, request);
      
      if (operationResult->returnCode == 0)
      {
         printf("Server version: %s, Return code: '%d', Return message '%s'\n", JS_CSTR( operationResult->version),  operationResult->returnCode, JS_CSTR( operationResult->returnMessage ));
         printf("Repository directory: %s\n\n", JS_CSTR( request->resource->uriString));
         fflush( stdout );
         
         for (res = operationResult->resources; res; res = res->next)
         {
            // Print the list of found objects in the specified directory
            printf("%s%s \t%s\n", ((!xmlStrcmp(JS_UTFSTR( res->wsType ), JS_TYPE_FOLDER )) ? "<DIR> " : ""), JS_UTFSTR( res->name ), JS_UTFSTR( res->label ));
         }
      }
      else
      {
          printf("Error executing the service: %d (%s)", operationResult->returnCode, JS_UTFSTR(operationResult->returnMessage) );
      
      }
                                
      // Clean up the allocated resources....      
      jasperserver_server_free( jasperServer );
      jasperserver_request_free( request );
      jasperserver_operation_result_free( operationResult );
      
}


int main(int argc, char **argv)
{ 
   char *dir = "/";
   printf("JasperServer list sample (C) 2007 JasperSoft Corp.!\n");
   
   if (argc > 1)
   {
     dir = argv[1];  
   }
   else
   {
     printf("Repository path not specified. Using root (/) as default.\n\n");
   }
   list_directory(dir);

  return 0;
}
